from dataclasses import dataclass, field
from enum import Enum


class DataUnits(Enum):
    """
    For conversion from KB to other (binary) units
    """

    KB = 1
    MB = 1024
    GB = 1024 ** 2
    TB = 1024 ** 3


class ServiceStatus(Enum):
    OK = "ok"
    DOWN = "down"
    CRITICAL = "critical"
    WARNING = "warning"
    DISABLED = "disabled"
    BACKUP_IN_PROGRESS = "backup_in_progress"
    BACKUP_STARTED = "backup_started"

    @property
    def codename(self):
        return self.value


class CPULoadReason(Enum):
    OTHER = "unknown"
    MYSQL = "mysql"


def convert_data(value: int, use_bytes: bool = False) -> str:
    if use_bytes:
        value = value * 1.00 / 1024
    if value < 1024:
        factor = DataUnits.KB
    elif 1024 <= value < 1024 ** 2:
        factor = DataUnits.MB
    elif 1024 ** 2 <= value < 1024 ** 3:
        factor = DataUnits.GB
    else:
        factor = DataUnits.TB

    return f"{round(value * 1.00 / factor.value, 2)} {factor.name}"


@dataclass
class Result:
    success: bool = True
    data: dict = field(default_factory=dict)
    message: str = ""
    warnings: list = field(default_factory=list)


class ScriptStatus(Enum):
    TIMEOUT = "timeout"
    STARTED = "started"
    IN_PROGRESS = "progress"
    CANNOT_START = "start_failed"
    SKIP = "skipped"

    @property
    def codename(self):
        return self.value
