import logging
import os
import socket
import sys

from dotenv import load_dotenv

default_root = "/root/monitoring_alert_checkers"

# Logging config
LOG_FILE = f'/root/{os.getenv("LOG_FILE", "managed_servers.log")}'
logger = logging.getLogger("managed_servers")
logger.setLevel(logging.INFO)
log_format = (
    f"%(asctime)s {socket.gethostname()} [alert_fixer - %(levelname)s]: %(message)s"
)

file_handler = logging.FileHandler(LOG_FILE)
file_handler.setLevel(logging.INFO)
file_handler.setFormatter(logging.Formatter(log_format))

console_handler = logging.StreamHandler(sys.stdout)
console_handler.setLevel(logging.INFO)
console_handler.setFormatter(logging.Formatter(log_format))

logger.addHandler(file_handler)
logger.addHandler(console_handler)


env_file = os.getenv("ENV_FILE", "/etc/service_env")
load_dotenv(env_file)
logger.info(f"ENV loaded from: {env_file}")

ALERTS_DIR = f'{os.getenv("PROJECT_ROOT", default_root)}/scripts'
CHECK_LOCKS_DIR = f'{os.getenv("PROJECT_ROOT", default_root)}/var/check_locks'

MAIN_SCRIPT = "service_alerts.sh"
SERVER_MANAGEMENT_TYPE = os.getenv("MANAGEMENT_TYPE")
BASH_UTILS = f'{os.getenv("PROJECT_ROOT", default_root)}/utils.sh'
TEMP_ACKNOWLEDGE_DURATION = 24 * 3600
